package cn.estsh.i3plus.pojo.base.enumutil;

import com.fasterxml.jackson.annotation.JsonFormat;
import org.apache.commons.lang3.StringUtils;

/**
 * @Description : 软适配 枚举接口
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2019-08-13 9:34
 * @Modify:
 **/
public class BlockSoftSwitchEnumUtil {

    /**
     * 套件类型
     */
    @JsonFormat(shape = JsonFormat.Shape.OBJECT)
    public enum CASE_TYPE{
        SOCKET(10,"SOCKET"),
        RESTFUL(20,"RESTFUL"),
        DATASOURCE(30,"数据源"),
        WEBSERVICE(40,"WebService"),
        MQ(50,"消息队列"),
        WebSocket(60,"WebSocket");

        private int value;
        private String description;

        CASE_TYPE(int value, String description) {
            this.value = value;
            this.description = description;
        }

        public int getValue() {
            return value;
        }

        public String getDescription() {
            return description;
        }

        public static String valueOfDescription(int val) {
            String tmp = null;
            for (int i = 0; i < values().length; i++) {
                if (values()[i].value == val) {
                    tmp = values()[i].description;
                }
            }
            return tmp;
        }

        public static CASE_TYPE valueOf(int val) {
            String tmp = null;
            for (int i = 0; i < values().length; i++) {
                if (values()[i].value == val) {
                    return values()[i];
                }
            }
            return null;
        }
    }

    /**
     * 请求来源
     */
    @JsonFormat(shape = JsonFormat.Shape.OBJECT)
    public enum REQUEST_SOURCE{
        ACTIVE(1,"主动请求"),
        SCHEDULE(2,"定时调度"),
        RABBITMQ(3,"MQ 调用");

        private int value;
        private String description;

        REQUEST_SOURCE(int value, String description) {
            this.value = value;
            this.description = description;
        }

        public int getValue() {
            return value;
        }

        public String getDescription() {
            return description;
        }

        public static String valueOfDescription(int val) {
            String tmp = null;
            for (int i = 0; i < values().length; i++) {
                if (values()[i].value == val) {
                    tmp = values()[i].description;
                }
            }
            return tmp;
        }
    }

    /**
     * 认证方式
     */
    @JsonFormat(shape = JsonFormat.Shape.OBJECT)
    public enum VERIFICATION_METHOD{
        LOGIN(1,"登录");

        private int value;
        private String description;

        VERIFICATION_METHOD(int value, String description) {
            this.value = value;
            this.description = description;
        }

        public int getValue() {
            return value;
        }

        public String getDescription() {
            return description;
        }

        public static String valueOfDescription(int val) {
            String tmp = null;
            for (int i = 0; i < values().length; i++) {
                if (values()[i].value == val) {
                    tmp = values()[i].description;
                }
            }
            return tmp;
        }
    }

    /**
     * 认证类型
     */
    @JsonFormat(shape = JsonFormat.Shape.OBJECT)
    public enum KEY_TYPE{
        ACCOUNT_PASSWORD(1,"账号密码");

        private int value;
        private String description;

        KEY_TYPE(int value, String description) {
            this.value = value;
            this.description = description;
        }

        public int getValue() {
            return value;
        }

        public String getDescription() {
            return description;
        }

        public static String valueOfDescription(int val) {
            String tmp = null;
            for (int i = 0; i < values().length; i++) {
                if (values()[i].value == val) {
                    tmp = values()[i].description;
                }
            }
            return tmp;
        }
    }

    /**
     * 数据库连接方式
     */
    @JsonFormat(shape = JsonFormat.Shape.OBJECT)
    public enum DATA_SOURCE_TYPE {
        SOURCE_MARIA_DB(100, "MariaDB", "MariaDB 10.1","com.mysql.jdbc.Driver",3306,null),
        SOURCE_SQL_SERVER(200, "SQL Server", "SQL Server 2017","com.microsoft.sqlserver.jdbc.SQLServerDriver",1433,"dbo"),
        SOURCE_ORACLE(300, "Oracle", "Oralce 12C","oracle.jdbc.driver.OracleDriver",1521,null),
        SOURCE_POSTGRE_SQL(400, "PostgreSql", "PostgreSql 10.5","org.postgresql.Driver",5432,"public");

        private int value;
        private String code;
        private String description;
        private String driverClassName;
        private int defaultPort;
        private String defaultSchemaPattern;

        private DATA_SOURCE_TYPE (int value, String code, String description,String driverClassName,int port,String defaultSchemaPattern) {
            this.value = value;
            this.code = code;
            this.description = description;
            this.driverClassName = driverClassName;
            this.defaultPort = port;
            this.defaultSchemaPattern = defaultSchemaPattern;
        }

        public int getValue() {
            return value;
        }

        public String getCode() {
            return code;
        }

        public String getDescription() {
            return description;
        }

        public String getDriverClassName() {
            return driverClassName;
        }

        public int getDefaultPort() {
            return defaultPort;
        }

        public String getDefaultSchemaPattern() {
            return defaultSchemaPattern;
        }

        public static String valueOfCode(int val) {
            String tmp = null;
            for (int i = 0; i < values().length; i++) {
                if (values()[i].value == val) {
                    tmp = values()[i].code;
                }
            }
            return tmp;
        }

        public static int codeOfValue(String code) {
            int tmp = 1;
            for (int i = 0; i < values().length; i++) {
                if (values()[i].code.equals(code)) {
                    tmp = values()[i].value;
                }
            }
            return tmp;
        }

        public static String valueOfDescription(int val) {
            String tmp = null;
            for (int i = 0; i < values().length; i++) {
                if (values()[i].value == val) {
                    tmp = values()[i].description;
                }
            }
            return tmp;
        }

        public static BlockSoftSwitchEnumUtil.DATA_SOURCE_TYPE valueOf(int val) {
            String tmp = null;
            for (int i = 0; i < values().length; i++) {
                if (values()[i].value == val) {
                    return values()[i];
                }
            }
            return null;
        }

        public static String codeOfDescription(String code) {
            String tmp = null;
            for (int i = 0; i < values().length; i++) {
                if (values()[i].code.equals(code)) {
                    tmp = values()[i].description;
                }
            }
            return tmp;
        }

        public String getJDBCUrl(String database,String host,Integer port){
            if(this.getValue() == SOURCE_MARIA_DB.getValue()){
                return getJDBCUrlMySQL(database,host,port);
            }else if(this.getValue() == SOURCE_ORACLE.getValue()){
                return getJDBCUrlOracle(database,host,port);
            }else if(this.getValue() == SOURCE_POSTGRE_SQL.getValue()){
                return getJDBCUrlPostgreSQL(database,host,port);
            }else if(this.getValue() == SOURCE_SQL_SERVER.getValue()){
                return getJDBCUrlSQLServer(database,host,port);
            }
            return null;
        }

        public static BlockSoftSwitchEnumUtil.DATA_SOURCE_TYPE getDataSourceURL(String databaseProductName){
            if(StringUtils.isNotBlank(databaseProductName)){
                if(databaseProductName.indexOf(":mysql:") != -1){
                    return SOURCE_MARIA_DB;
                }else if(databaseProductName.indexOf(":oracle:") != -1){
                    return SOURCE_ORACLE;
                }else if(databaseProductName.indexOf(":postgresql:") != -1){
                    return SOURCE_POSTGRE_SQL;
                }else if(databaseProductName.indexOf(":sqlserver:") != -1){
                    return SOURCE_SQL_SERVER;
                }
            }
            return null;
        }

        private String getJDBCUrlMySQL(String database,String host,Integer port){
            return "jdbc:mysql://"+host+":"+port+"/"+database+"?autoReconnect=true&useSSL=false&characterEncoding=utf-8";
        }

        private String getJDBCUrlOracle(String database,String host,Integer port){
            return "jdbc:oracle:thin:@"+host+":"+port+":"+database;
        }

        private String getJDBCUrlPostgreSQL(String database,String host,Integer port){
            return "jdbc:postgresql://"+host+":"+port+"/"+database;
        }

        private String getJDBCUrlSQLServer(String database,String host,Integer port){
            return "jdbc:sqlserver://" + host + ":" + port + ";database=" + database;
        }
    }

    /**
     * 参数值类型
     */
    @JsonFormat(shape = JsonFormat.Shape.OBJECT)
    public enum PARAM_VALUE_TYPE{
        NUM(10,"数字"),
        STRING(20,"字符串"),
        BOOLEAN(30,"布尔"),
        MAP(40,"字典"),
        LIST(50,"列表");

        private int value;
        private String description;

        PARAM_VALUE_TYPE(int value, String description) {
            this.value = value;
            this.description = description;
        }

        public int getValue() {
            return value;
        }

        public String getDescription() {
            return description;
        }

        public static String valueOfDescription(int val) {
            String tmp = null;
            for (int i = 0; i < values().length; i++) {
                if (values()[i].value == val) {
                    tmp = values()[i].description;
                }
            }
            return tmp;
        }
    }

    /**
     * 参数类型
     */
    @JsonFormat(shape = JsonFormat.Shape.OBJECT)
    public enum PARAM_TYPE{
        OUT_PARAM(1,"出参"),
        IN_PARAM(2,"入参"),
        REQUEST_HEADER(3,"请求头");

        private int value;
        private String description;

        PARAM_TYPE(int value, String description) {
            this.value = value;
            this.description = description;
        }

        public int getValue() {
            return value;
        }

        public String getDescription() {
            return description;
        }

        public static String valueOfDescription(int val) {
            String tmp = null;
            for (int i = 0; i < values().length; i++) {
                if (values()[i].value == val) {
                    tmp = values()[i].description;
                }
            }
            return tmp;
        }
    }

    /**
     * 适配模式
     */
    @JsonFormat(shape = JsonFormat.Shape.OBJECT)
    public enum SUIT_MODE{
        ACTIVE(1,"客户端"),
        PASSIVE(2,"服务端");

        private int value;
        private String description;

        SUIT_MODE(int value, String description) {
            this.value = value;
            this.description = description;
        }

        public int getValue() {
            return value;
        }

        public String getDescription() {
            return description;
        }

        public static String valueOfDescription(int val) {
            String tmp = null;
            for (int i = 0; i < values().length; i++) {
                if (values()[i].value == val) {
                    tmp = values()[i].description;
                }
            }
            return tmp;
        }
    }

    /**
     * 适配方式
     */
    @JsonFormat(shape = JsonFormat.Shape.OBJECT)
    public enum SUIT_METHOD{
        ACTIVE(1,"主动适配"),
        SCHEDULE(2,"定时调度");

        private int value;
        private String description;

        SUIT_METHOD(int value, String description) {
            this.value = value;
            this.description = description;
        }

        public int getValue() {
            return value;
        }

        public String getDescription() {
            return description;
        }

        public static String valueOfDescription(int val) {
            String tmp = null;
            for (int i = 0; i < values().length; i++) {
                if (values()[i].value == val) {
                    tmp = values()[i].description;
                }
            }
            return tmp;
        }
    }

    /**
     * 数据库操作方式
     */
    @JsonFormat(shape = JsonFormat.Shape.OBJECT)
    public enum DATABASE_OPERATE_TYPE{
        READ(1,"读取"),
        WRITE(2,"写入");

        private int value;
        private String description;

        DATABASE_OPERATE_TYPE(int value, String description) {
            this.value = value;
            this.description = description;
        }

        public int getValue() {
            return value;
        }

        public String getDescription() {
            return description;
        }

        public static String valueOfDescription(int val) {
            String tmp = null;
            for (int i = 0; i < values().length; i++) {
                if (values()[i].value == val) {
                    tmp = values()[i].description;
                }
            }
            return tmp;
        }
    }

    /**
     * 适配方式
     */
    @JsonFormat(shape = JsonFormat.Shape.OBJECT)
    public enum SUIT_SERVER_WEB_SERVICE{
        ACTIVE(40001,"serverWebServiceHello","hello"),
        SCHEDULE(40002,"serverWebServiceSendEmail","邮件测试");

        private int value;
        private String description;
        private String clazzName;

        SUIT_SERVER_WEB_SERVICE(int value, String clazzName,String description) {
            this.value = value;
            this.clazzName = clazzName;
            this.description = description;
        }

        public int getValue() {
            return value;
        }

        public String getDescription() {
            return description;
        }

        public static String valueOfDescription(int val) {
            String tmp = null;
            for (int i = 0; i < values().length; i++) {
                if (values()[i].value == val) {
                    tmp = values()[i].description;
                }
            }
            return tmp;
        }
    }

}
